import 'package:decorated_flutter/decorated_flutter.dart';
import 'package:flutter/material.dart';
import 'package:oktoast/oktoast.dart';
import 'package:tim_fluttify_example/src/bloc/bloc.export.dart';
import 'package:tim_fluttify_example/src/di/injector.dart';
import 'package:tim_fluttify_example/src/resource/R.dart';
import 'package:tim_fluttify_example/src/ui/screen/screen.export.dart';

final gNavigatorKey = GlobalKey<NavigatorState>();

class TimApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BLoCProvider<AppBLoC>(
      bloc: Injector.obtain(),
      child: OKToast(
        duration: Duration(seconds: 2),
        child: MaterialApp(
          navigatorKey: gNavigatorKey,
          onGenerateRoute: _onGenerateRoute,
        ),
      ),
    );
  }

  Route _onGenerateRoute(RouteSettings settings) {
    final path = settings.name;
    final arguments = settings.arguments as Map<String, dynamic>;

    switch (path) {
      // 登录
      case Routes.login:
        return DecoratedRoute<LoginBLoC, void>(
          screen: LoginScreen(),
          bloc: Injector.obtain(),
          init: (bloc) {},
          routeName: path,
        );
      // 会话列表
      case Routes.home:
        return DecoratedRoute<HomeBLoC, void>(
          screen: SessionsScreen(),
          bloc: Injector.obtain(),
          init: (bloc) {
            bloc..sessionList.update()..friendList.update();
          },
          routeName: path,
        );
      // 添加好友
      case Routes.add_friend:
        return DecoratedRoute<AddFriendBLoC, void>(
          screen: AddFriendScreen(),
          bloc: Injector.obtain(),
          init: (bloc) {},
          routeName: path,
        );
      default:
        return DecoratedRoute(screen: UnknownScreen(), routeName: path);
    }
  }
}
