// ignore_for_file: non_constant_identifier_names
/// 领域模型
///
/// 直接对应服务器返回的数据结构. 领域模型不应该直接与视图层(widget)解除, 而应先转换为视图模式
/// 再与视图层交互.

import 'dart:convert';

/// 消息壳
class Bean {
  /// data可能为对象或者列表
  dynamic data;

  /// 错误信息
  ///
  /// 如果不为null, 那么必有 {message:'xxx', code:xxx, 其他字段}.
  /// 如果 code 为403 需要做清除token 跳转登陆页is_vip
  Map<String, dynamic> error;

  /// 分页信息
  Map<String, dynamic> meta;

  Bean._({this.error, this.data, this.meta});

  static Bean fromJson(Map<String, dynamic> json) {
    return Bean._(
      error: json['error'] as Map<String, dynamic>,
      data: json['data'], // 可能为list, 可能为map
      meta: json['meta'] as Map<String, dynamic>,
    );
  }

  Map<String, Object> toJson() {
    final Map<String, Object> data = new Map<String, Object>();
    data['error'] = this.error;
    return data;
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 分页信息
class Meta {
  int current_page;
  int from;
  int last_page;
  String path;
  int per_page;
  int to;
  int total;

  Meta({
    this.current_page,
    this.from,
    this.last_page,
    this.path,
    this.per_page,
    this.to,
    this.total,
  });

  static Meta fromJson(Map<String, dynamic> json) {
    final bean = Meta();
    bean.current_page = json['current_page'] as int;
    bean.from = json['from'] as int;
    bean.last_page = json['last_page'] as int;
    bean.path = json['path'] as String;
    bean.per_page = json['per_page'] as int;
    bean.to = json['to'] as int;
    bean.total = json['total'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'current_page': current_page,
      'from': from,
      'last_page': last_page,
      'path': path,
      'per_page': per_page,
      'to': to,
      'total': total,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 登录
class Login {
  String access_token;
  String token_type;
  int expires_in;
  int user_id;
  String userSig;

  Login({
    this.access_token,
    this.token_type,
    this.expires_in,
    this.user_id,
    this.userSig,
  });

  static Login fromJson(Map<String, dynamic> json) {
    final bean = Login();
    bean.access_token = json['access_token'] as String;
    bean.token_type = json['token_type'] as String;
    bean.expires_in = json['expires_in'] as int;
    bean.user_id = json['user_id'] as int;
    bean.userSig = json['userSig'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'access_token': access_token,
      'token_type': token_type,
      'expires_in': expires_in,
      'user_id': user_id,
      'userSig': userSig,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// banner
class Banner {
  String link;
  String image;
  Null remark;

  Banner({
    this.link,
    this.image,
    this.remark,
  });

  static Banner fromJson(Map<String, dynamic> json) {
    final bean = Banner();
    bean.link = json['link'] as String;
    bean.image = json['image'] as String;
    bean.remark = json['remark'] as Null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'link': link,
      'image': image,
      'remark': remark,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 我的话题
class MyTopic {
  String followed_at;
  Topic topic;

  MyTopic({
    this.followed_at,
    this.topic,
  });

  static MyTopic fromJson(Map<String, dynamic> json) {
    final bean = MyTopic();
    bean.followed_at = json['followed_at'] as String;
    bean.topic = json['topic'] != null ? Topic.fromJson(json['topic']) : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'followed_at': followed_at,
      'topic': topic,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 话题
class Topic {
  int id;
  String name;
  int sort;
  String thumb;
  String summary;
  int category_id;
  int is_hot;
  int is_followed;
  List<Author> hot_authors;
  int news_amount;
  int join_amount;
  int followers_amount;

  Topic({
    this.id,
    this.name,
    this.sort,
    this.thumb,
    this.summary,
    this.category_id,
    this.is_hot,
    this.is_followed,
    this.hot_authors,
    this.news_amount,
    this.join_amount,
    this.followers_amount,
  });

  static Topic fromJson(Map<String, dynamic> json) {
    final bean = Topic();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.sort = json['sort'] as int;
    bean.thumb = json['thumb'] as String;
    bean.summary = json['summary'] as String;
    bean.category_id = json['category_id'] as int;
    bean.is_hot = json['is_hot'] as int;
    bean.is_followed = json['is_followed'] as int;
    if (json['hot_authors'] != null) {
      bean.hot_authors = List<Author>();
      json['hot_authors'].forEach((v) {
        bean.hot_authors.add(Author.fromJson(v as Map<String, Object>));
      });
    }
    bean.news_amount = json['news_amount'] as int;
    bean.join_amount = json['join_amount'] as int;
    bean.followers_amount = json['followers_amount'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'sort': sort,
      'thumb': thumb,
      'summary': summary,
      'category_id': category_id,
      'is_hot': is_hot,
      'is_followed': is_followed,
      'hot_authors': hot_authors,
      'news_amount': news_amount,
      'join_amount': join_amount,
      'followers_amount': followers_amount,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 话题详情
class TopicDetail {
  int id;
  String name;
  int sort;
  String thumb;
  String summary;
  int category_id;
  int is_hot;
  int is_followed;
  List<Author> hot_authors;
  int news_amount;
  int followers_amount;
  TopicCategory category;

  TopicDetail({
    this.id,
    this.name,
    this.sort,
    this.thumb,
    this.summary,
    this.category_id,
    this.is_hot,
    this.is_followed,
    this.hot_authors,
    this.news_amount,
    this.followers_amount,
    this.category,
  });

  static TopicDetail fromJson(Map<String, dynamic> json) {
    final bean = TopicDetail();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.sort = json['sort'] as int;
    bean.thumb = json['thumb'] as String;
    bean.summary = json['summary'] as String;
    bean.category_id = json['category_id'] as int;
    bean.is_hot = json['is_hot'] as int;
    bean.is_followed = json['is_followed'] as int;
    if (json['hot_authors'] != null) {
      bean.hot_authors = List<Author>();
      json['hot_authors'].forEach((v) {
        bean.hot_authors.add(Author.fromJson(v as Map<String, Object>));
      });
    }
    bean.news_amount = json['news_amount'] as int;
    bean.followers_amount = json['followers_amount'] as int;
    bean.category = json['category'] != null
        ? TopicCategory.fromJson(json['category'])
        : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'sort': sort,
      'thumb': thumb,
      'summary': summary,
      'category_id': category_id,
      'is_hot': is_hot,
      'is_followed': is_followed,
      'hot_authors': hot_authors,
      'news_amount': news_amount,
      'followers_amount': followers_amount,
      'category': category,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 作者
class Author {
  int news_count;
  User user;

  Author({
    this.news_count,
    this.user,
  });

  static Author fromJson(Map<String, dynamic> json) {
    final bean = Author();
    bean.news_count = json['news_count'] as int;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'news_count': news_count,
      'user': user,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 图形验证码
class GraphicalCaptcha {
  String base64;
  String uuid;

  GraphicalCaptcha({
    this.base64,
    this.uuid,
  });

  static GraphicalCaptcha fromJson(Map<String, dynamic> json) {
    final bean = GraphicalCaptcha();
    bean.base64 = json['base64'] as String;
    bean.uuid = json['uuid'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'base64': base64,
      'uuid': uuid,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 动态
class Activity {
  int id;
  int user_id;
  int topic_id;
  String content;
  List<String> pictures;
  Link link;
  num lat;
  num lng;
  int read_num;
  int good_count;
  num distance;
  int comments_amount;
  String created_at;
  int is_good;
  Topic topic;
  User user;
  List<Comment> comments;

  Activity({
    this.id,
    this.user_id,
    this.topic_id,
    this.content,
    this.pictures,
    this.link,
    this.lat,
    this.lng,
    this.read_num,
    this.good_count,
    this.distance,
    this.comments_amount,
    this.created_at,
    this.is_good,
    this.topic,
    this.user,
    this.comments,
  });

  static Activity fromJson(Map<String, dynamic> json) {
    final bean = Activity();
    bean.id = json['id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.topic_id = json['topic_id'] as int;
    bean.content = json['content'] as String;
    bean.pictures = (json['pictures'] as List)?.cast<String>() ?? [];
    bean.link = json['link'] != null ? Link.fromJson(json['link']) : null;
    bean.lat = json['lat'] as num;
    bean.lng = json['lng'] as num;
    bean.read_num = json['read_num'] as int;
    bean.good_count = json['good_count'] as int;
    bean.distance = json['distance'] as num;
    bean.comments_amount = json['comments_amount'] as int;
    bean.created_at = json['created_at'] as String;
    bean.is_good = json['is_good'] as int;
    bean.topic = json['topic'] != null ? Topic.fromJson(json['topic']) : null;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    if (json['comments'] != null) {
      bean.comments = List<Comment>();
      json['comments'].forEach((v) {
        bean.comments.add(Comment.fromJson(v as Map<String, Object>));
      });
    }
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'user_id': user_id,
      'topic_id': topic_id,
      'content': content,
      'pictures': pictures,
      'link': link,
      'lat': lat,
      'lng': lng,
      'read_num': read_num,
      'good_count': good_count,
      'distance': distance,
      'comments_amount': comments_amount,
      'created_at': created_at,
      'is_good': is_good,
      'topic': topic,
      'user': user,
      'comments': comments,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 用户信息
class Profile {
  Company company;
  Profession profession;
  Business business;

  Profile({
    this.company,
    this.profession,
    this.business,
  });

  static Profile fromJson(Map<String, dynamic> json) {
    final bean = Profile();
    bean.company =
        json['company'] != null ? Company.fromJson(json['company']) : null;
    bean.profession = json['profession'] != null
        ? Profession.fromJson(json['profession'])
        : null;
    bean.business =
        json['business'] != null ? Business.fromJson(json['business']) : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'company': company,
      'profession': profession,
      'business': business,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 公司
class Company {
  int id;
  String name;

  Company({
    this.id,
    this.name,
  });

  static Company fromJson(Map<String, dynamic> json) {
    final bean = Company();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 职位
class Profession {
  int id;
  String name;

  Profession({
    this.id,
    this.name,
  });

  static Profession fromJson(Map<String, dynamic> json) {
    final bean = Profession();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 行业
class Business {
  int id;
  String name;
  String full_name;
  int sort;

  Business({
    this.id,
    this.name,
    this.sort,
  });

  static Business fromJson(Map<String, dynamic> json) {
    final bean = Business();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.full_name = json['full_name'] as String;
    bean.sort = json['sort'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'full_name': full_name,
      'sort': sort,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 链接
class Link {
  String url;
  String title;
  String image;

  Link({
    this.url,
    this.title,
    this.image,
  });

  static Link fromJson(Map<String, dynamic> json) {
    final bean = Link();
    bean.url = json['url'] as String;
    bean.title = json['title'] as String;
    bean.image = json['image'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'url': url,
      'title': title,
      'image': image,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 测试用短信验证码
class TestCaptcha {
  int interval;
  String code;

  TestCaptcha({
    this.interval,
    this.code,
  });

  static TestCaptcha fromJson(Map<String, dynamic> json) {
    final bean = TestCaptcha();
    bean.interval = json['interval'] as int;
    bean.code = json['code'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'interval': interval,
      'code': code,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 话题分类
class TopicCategory {
  int id;
  String name;
  int sort;
  int is_hot;
  int topics_amount;
  List<Topic> topics;

  TopicCategory({
    this.id,
    this.name,
    this.sort,
    this.is_hot,
    this.topics_amount,
    this.topics,
  });

  static TopicCategory fromJson(Map<String, dynamic> json) {
    final bean = TopicCategory();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.sort = json['sort'] as int;
    bean.is_hot = json['is_hot'] as int;
    bean.topics_amount = json['topics_amount'] as int;
    if (json['topics'] != null) {
      bean.topics = List<Topic>();
      json['topics'].forEach((v) {
        bean.topics.add(Topic.fromJson(v as Map<String, Object>));
      });
    }
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'sort': sort,
      'is_hot': is_hot,
      'topics_amount': topics_amount,
      'topics': topics,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 动态详情
class ActivityDetail {
  int id;
  int user_id;
  int topic_id;
  String content;
  List<String> pictures;
  Link link;
  num lat;
  num lng;
  int read_num;
  int good_count;
  num distance;
  int comments_amount;
  String created_at;
  int is_good;
  Topic topic;
  User user;
  List<String> thumb_pictures;

  ActivityDetail({
    this.id,
    this.user_id,
    this.topic_id,
    this.content,
    this.pictures,
    this.link,
    this.lat,
    this.lng,
    this.read_num,
    this.good_count,
    this.distance,
    this.comments_amount,
    this.created_at,
    this.is_good,
    this.topic,
    this.user,
    this.thumb_pictures,
  });

  static ActivityDetail fromJson(Map<String, dynamic> json) {
    final bean = ActivityDetail();
    bean.id = json['id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.topic_id = json['topic_id'] as int;
    bean.content = json['content'] as String;
    bean.pictures = (json['pictures'] as List)?.cast<String>() ?? [];
    bean.link = json['link'] != null ? Link.fromJson(json['link']) : null;
    bean.lat = json['lat'] as num;
    bean.lng = json['lng'] as num;
    bean.read_num = json['read_num'] as int;
    bean.good_count = json['good_count'] as int;
    bean.distance = json['distance'] as int;
    bean.comments_amount = json['comments_amount'] as int;
    bean.created_at = json['created_at'] as String;
    bean.is_good = json['is_good'] as int;
    bean.topic = json['topic'] != null ? Topic.fromJson(json['topic']) : null;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    bean.thumb_pictures =
        (json['thumb_pictures'] as List)?.cast<String>() ?? [];
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'user_id': user_id,
      'topic_id': topic_id,
      'content': content,
      'pictures': pictures,
      'link': link.toJson(),
      'lat': lat,
      'lng': lng,
      'read_num': read_num,
      'good_count': good_count,
      'distance': distance,
      'comments_amount': comments_amount,
      'created_at': created_at,
      'is_good': is_good,
      'topic': topic,
      'user': user,
      'thumb_pictures': thumb_pictures,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 动态评论
class Comment {
  int id;
  int news_id;
  int user_id;
  int reply_id;
  int to_user_id;
  String content;
  List<String> pictures;
  Link link;
  int good_count;
  int replies_amount;
  int is_good;
  String created_at;
  User user;
  List<Reply> replies;
  List<String> thumb_pictures;

  Comment({
    this.id,
    this.news_id,
    this.user_id,
    this.reply_id,
    this.to_user_id,
    this.content,
    this.pictures,
    this.link,
    this.good_count,
    this.replies_amount,
    this.is_good,
    this.created_at,
    this.user,
    this.replies,
    this.thumb_pictures,
  });

  static Comment fromJson(Map<String, dynamic> json) {
    final bean = Comment();
    bean.id = json['id'] as int;
    bean.news_id = json['news_id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.reply_id = json['reply_id'] as int;
    bean.to_user_id = json['to_user_id'] as int;
    bean.content = json['content'] as String;
    bean.pictures = (json['pictures'] as List)?.cast<String>() ?? [];
    bean.link = json['link'] as Link;
    bean.good_count = json['good_count'] as int;
    bean.replies_amount = json['replies_amount'] as int;
    bean.is_good = json['is_good'] as int;
    bean.created_at = json['created_at'] as String;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    if (json['replies'] != null) {
      bean.replies = List<Reply>();
      json['replies'].forEach((v) {
        bean.replies.add(Reply.fromJson(v as Map<String, Object>));
      });
    }
    bean.thumb_pictures =
        (json['thumb_pictures'] as List)?.cast<String>() ?? [];
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'news_id': news_id,
      'user_id': user_id,
      'reply_id': reply_id,
      'to_user_id': to_user_id,
      'content': content,
      'pictures': pictures,
      'link': link,
      'good_count': good_count,
      'replies_amount': replies_amount,
      'is_good': is_good,
      'created_at': created_at,
      'user': user,
      'replies': replies,
      'thumb_pictures': thumb_pictures,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 评论回复
class Reply {
  int id;
  int news_id;
  int user_id;
  int reply_id;
  int to_user_id;
  String content;
  List<String> pictures;
  Link link;
  int good_count;
  int replies_amount;
  int is_good;
  String created_at;
  User user;
  List<String> thumb_pictures;

  Reply({
    this.id,
    this.news_id,
    this.user_id,
    this.reply_id,
    this.to_user_id,
    this.content,
    this.pictures,
    this.link,
    this.good_count,
    this.replies_amount,
    this.is_good,
    this.created_at,
    this.user,
    this.thumb_pictures,
  });

  static Reply fromJson(Map<String, dynamic> json) {
    final bean = Reply();
    bean.id = json['id'] as int;
    bean.news_id = json['news_id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.reply_id = json['reply_id'] as int;
    bean.to_user_id = json['to_user_id'] as int;
    bean.content = json['content'] as String;
    bean.pictures = (json['pictures'] as List)?.cast<String>() ?? [];
    bean.link = json['link'] as Link;
    bean.good_count = json['good_count'] as int;
    bean.replies_amount = json['replies_amount'] as int;
    bean.is_good = json['is_good'] as int;
    bean.created_at = json['created_at'] as String;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    bean.thumb_pictures =
        (json['thumb_pictures'] as List)?.cast<String>() ?? [];
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'news_id': news_id,
      'user_id': user_id,
      'reply_id': reply_id,
      'to_user_id': to_user_id,
      'content': content,
      'pictures': pictures,
      'link': link,
      'good_count': good_count,
      'replies_amount': replies_amount,
      'is_good': is_good,
      'created_at': created_at,
      'user': user,
      'thumb_pictures': thumb_pictures,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 照片上传结果
class PictureUpload {
  String url;

  PictureUpload({
    this.url,
  });

  static PictureUpload fromJson(Map<String, dynamic> json) {
    final bean = PictureUpload();
    bean.url = json['url'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'url': url,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 直播分类
class LiveCategory {
  int id;
  String name;
  String image;
  int sort;
  int is_hot;
  String description;
  int type;
  int auto_thumb;
  String created_at;
  String updated_at;

  LiveCategory({
    this.id,
    this.name,
    this.image,
    this.sort,
    this.is_hot,
    this.description,
    this.type,
    this.auto_thumb,
    this.created_at,
    this.updated_at,
  });

  static LiveCategory fromJson(Map<String, dynamic> json) {
    final bean = LiveCategory();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.image = json['image'] as String;
    bean.sort = json['sort'] as int;
    bean.is_hot = json['is_hot'] as int;
    bean.description = json['description'] as String;
    bean.type = json['type'] as int;
    bean.auto_thumb = json['auto_thumb'] as int;
    bean.created_at = json['created_at'] as String;
    bean.updated_at = json['updated_at'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'image': image,
      'sort': sort,
      'is_hot': is_hot,
      'description': description,
      'type': type,
      'auto_thumb': auto_thumb,
      'created_at': created_at,
      'updated_at': updated_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 直播间
class Live {
  int id;
  String name;
  String room_no;
  String thumb;
  int audience_num;
  String announcement;
  int popularity;
  String introduction;
  String url;
  int is_living;
  int pk_id;
  int screen;
  int type;
  String start_time;
  String province;
  String city;
  String county;
  Null location;
  int category_id;
  String category_name;
  Anchor anchor;
  LiveCategory category;
  String play_url;

  Live({
    this.id,
    this.name,
    this.room_no,
    this.thumb,
    this.audience_num,
    this.announcement,
    this.popularity,
    this.introduction,
    this.url,
    this.is_living,
    this.pk_id,
    this.screen,
    this.type,
    this.start_time,
    this.province,
    this.city,
    this.county,
    this.location,
    this.category_id,
    this.category_name,
    this.anchor,
    this.category,
    this.play_url,
  });

  static Live fromJson(Map<String, dynamic> json) {
    final bean = Live();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.room_no = json['room_no'] as String;
    bean.thumb = json['thumb'] as String;
    bean.audience_num = json['audience_num'] as int;
    bean.announcement = json['announcement'] as String;
    bean.popularity = json['popularity'] as int;
    bean.introduction = json['introduction'] as String;
    bean.url = json['url'] as String;
    bean.is_living = json['is_living'] as int;
    bean.pk_id = json['pk_id'] as int;
    bean.screen = json['screen'] as int;
    bean.type = json['type'] as int;
    bean.start_time = json['start_time'] as String;
    bean.province = json['province'] as String;
    bean.city = json['city'] as String;
    bean.county = json['county'] as String;
    bean.location = json['location'] as Null;
    bean.category_id = json['category_id'] as int;
    bean.category_name = json['category_name'] as String;
    bean.anchor =
        json['anchor'] != null ? Anchor.fromJson(json['anchor']) : null;
    bean.category = json['category'] != null
        ? LiveCategory.fromJson(json['category'])
        : null;
    bean.play_url = json['play_url'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'room_no': room_no,
      'thumb': thumb,
      'audience_num': audience_num,
      'announcement': announcement,
      'popularity': popularity,
      'introduction': introduction,
      'url': url,
      'is_living': is_living,
      'pk_id': pk_id,
      'screen': screen,
      'type': type,
      'start_time': start_time,
      'province': province,
      'city': city,
      'county': county,
      'location': location,
      'category_id': category_id,
      'category_name': category_name,
      'anchor': anchor,
      'category': category,
      'play_url': play_url,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 主播
class Anchor {
  int id;
  String nickname;
  String avatar;
  int sex;
  String signature;
  String nice_id;
  int follow_num;
  int fans_num;
  int is_followed;
  int is_living;
  int level;
  int exp;
  int exp_max;
  Null earnings_gold;
  int earnings_diamond;

  Anchor({
    this.id,
    this.nickname,
    this.avatar,
    this.sex,
    this.signature,
    this.nice_id,
    this.follow_num,
    this.fans_num,
    this.is_followed,
    this.is_living,
    this.level,
    this.exp,
    this.exp_max,
    this.earnings_gold,
    this.earnings_diamond,
  });

  static Anchor fromJson(Map<String, dynamic> json) {
    final bean = Anchor();
    bean.id = json['id'] as int;
    bean.nickname = json['nickname'] as String;
    bean.avatar = json['avatar'] as String;
    bean.sex = json['sex'] as int;
    bean.signature = json['signature'] as String;
    bean.nice_id = json['nice_id'] as String;
    bean.follow_num = json['follow_num'] as int;
    bean.fans_num = json['fans_num'] as int;
    bean.is_followed = json['is_followed'] as int;
    bean.is_living = json['is_living'] as int;
    bean.level = json['level'] as int;
    bean.exp = json['exp'] as int;
    bean.exp_max = json['exp_max'] as int;
    bean.earnings_gold = json['earnings_gold'] as Null;
    bean.earnings_diamond = json['earnings_diamond'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'nickname': nickname,
      'avatar': avatar,
      'sex': sex,
      'signature': signature,
      'nice_id': nice_id,
      'follow_num': follow_num,
      'fans_num': fans_num,
      'is_followed': is_followed,
      'is_living': is_living,
      'level': level,
      'exp': exp,
      'exp_max': exp_max,
      'earnings_gold': earnings_gold,
      'earnings_diamond': earnings_diamond,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 好友
class Friend {
  int id;
  String nickname;
  int level;
  String level_badge;
  String level_image;
  int exp;
  String avatar;
  int fans_num;
  int follow_num;
  Null is_followed;
  String signature;
  String birthday;
  String province;
  String city;
  int common_friend_amount;

  Friend({
    this.id,
    this.nickname,
    this.level,
    this.level_badge,
    this.level_image,
    this.exp,
    this.avatar,
    this.fans_num,
    this.follow_num,
    this.is_followed,
    this.signature,
    this.birthday,
    this.province,
    this.city,
    this.common_friend_amount,
  });

  static Friend fromJson(Map<String, dynamic> json) {
    final bean = Friend();
    bean.id = json['id'] as int;
    bean.nickname = json['nickname'] as String;
    bean.level = json['level'] as int;
    bean.level_badge = json['level_badge'] as String;
    bean.level_image = json['level_image'] as String;
    bean.exp = json['exp'] as int;
    bean.avatar = json['avatar'] as String;
    bean.fans_num = json['fans_num'] as int;
    bean.follow_num = json['follow_num'] as int;
    bean.is_followed = json['is_followed'] as Null;
    bean.signature = json['signature'] as String;
    bean.birthday = json['birthday'] as String;
    bean.province = json['province'] as String;
    bean.city = json['city'] as String;
    bean.common_friend_amount = json['common_friend_amount'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'nickname': nickname,
      'level': level,
      'level_badge': level_badge,
      'level_image': level_image,
      'exp': exp,
      'avatar': avatar,
      'fans_num': fans_num,
      'follow_num': follow_num,
      'is_followed': is_followed,
      'signature': signature,
      'birthday': birthday,
      'province': province,
      'city': city,
      'common_friend_amount': common_friend_amount,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 当前用户信息
class User {
  int id;
  String nickname;
  String mobile;
  String username;
  String email;
  String avatar;
  int sex;
  String birthday;
  String signature;
  int verified;
  int is_anchor;
  int is_author;
  String province;
  String city;
  String invite_code;
  int integral;
  int is_super_admin;
  int gold;
  int diamond;
  int gold_consume;
  int diamond_consume;
  String earnings;
  String earnings_total;
  String nice_id;
  int follow_num;
  int is_followed;
  int fans_num;
  int follow_topic_num;
  int friend_num;
  String login_time;
  int level;
  int exp;
  int exp_max;
  String snowflake_id;
  String userSig;
  String address;
  String true_name;
  int is_vip;
  String user_vip_card;
  LiveRoom room;
  Profile profile;
  int common_friend_amount;

  User({
    this.id,
    this.nickname,
    this.mobile,
    this.username,
    this.email,
    this.avatar,
    this.sex,
    this.birthday,
    this.signature,
    this.verified,
    this.is_anchor,
    this.is_author,
    this.province,
    this.city,
    this.invite_code,
    this.integral,
    this.is_super_admin,
    this.gold,
    this.diamond,
    this.gold_consume,
    this.diamond_consume,
    this.earnings,
    this.earnings_total,
    this.nice_id,
    this.follow_num,
    this.is_followed,
    this.fans_num,
    this.follow_topic_num,
    this.friend_num,
    this.login_time,
    this.level,
    this.exp,
    this.exp_max,
    this.snowflake_id,
    this.userSig,
    this.address,
    this.true_name,
    this.is_vip,
    this.user_vip_card,
    this.room,
    this.profile,
    this.common_friend_amount,
  });

  static User fromJson(Map<String, dynamic> json) {
    final bean = User();
    bean.id = json['id'] as int;
    bean.nickname = json['nickname'] as String;
    bean.mobile = json['mobile'] as String;
    bean.username = json['username'] as String;
    bean.email = json['email'] as String;
    bean.avatar = json['avatar'] as String;
    bean.sex = json['sex'] as int;
    bean.birthday = json['birthday'] as String;
    bean.signature = json['signature'] as String;
    bean.verified = json['verified'] as int;
    bean.is_anchor = json['is_anchor'] as int;
    bean.is_author = json['is_author'] as int;
    bean.province = json['province'] as String;
    bean.city = json['city'] as String;
    bean.invite_code = json['invite_code'] as String;
    bean.integral = json['integral'] as int;
    bean.is_super_admin = json['is_super_admin'] as int;
    bean.gold = json['gold'] as int;
    bean.diamond = json['diamond'] as int;
    bean.gold_consume = json['gold_consume'] as int;
    bean.diamond_consume = json['diamond_consume'] as int;
    bean.earnings = json['earnings'] as String;
    bean.earnings_total = json['earnings_total'] as String;
    bean.nice_id = json['nice_id'] as String;
    bean.follow_num = json['follow_num'] as int;
    bean.is_followed = json['is_followed'] as int;
    bean.fans_num = json['fans_num'] as int;
    bean.follow_topic_num = json['follow_topic_num'] as int;
    bean.friend_num = json['friend_num'] as int;
    bean.login_time = json['login_time'] as Null;
    bean.level = json['level'] as int;
    bean.exp = json['exp'] as int;
    bean.exp_max = json['exp_max'] as int;
    bean.snowflake_id = json['snowflake_id'] as String;
    bean.userSig = json['userSig'] as String;
    bean.address = json['address'] as String;
    bean.true_name = json['true_name'] as String;
    bean.is_vip = json['is_vip'] as int;
    bean.user_vip_card = json['user_vip_card'] as String;
    bean.room = json['room'] != null ? LiveRoom.fromJson(json['room']) : null;
    bean.profile =
        json['profile'] != null ? Profile.fromJson(json['profile']) : null;
    bean.common_friend_amount = json['common_friend_amount'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'nickname': nickname,
      'mobile': mobile,
      'username': username,
      'email': email,
      'avatar': avatar,
      'sex': sex,
      'birthday': birthday,
      'signature': signature,
      'verified': verified,
      'is_anchor': is_anchor,
      'is_author': is_author,
      'province': province,
      'city': city,
      'invite_code': invite_code,
      'integral': integral,
      'is_super_admin': is_super_admin,
      'gold': gold,
      'diamond': diamond,
      'gold_consume': gold_consume,
      'diamond_consume': diamond_consume,
      'earnings': earnings,
      'earnings_total': earnings_total,
      'nice_id': nice_id,
      'follow_num': follow_num,
      'is_followed': is_followed,
      'fans_num': fans_num,
      'follow_topic_num': follow_topic_num,
      'friend_num': friend_num,
      'login_time': login_time,
      'level': level,
      'exp': exp,
      'exp_max': exp_max,
      'snowflake_id': snowflake_id,
      'userSig': userSig,
      'address': address,
      'true_name': true_name,
      'is_vip': is_vip,
      'user_vip_card': user_vip_card,
      'room': room,
      'profile': profile,
      'common_friend_amount': common_friend_amount,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 礼物
class Gift {
  int id;
  String name;
  String image;
  String gif;
  int price;
  int popularity;
  String luck_rate;
  int type;
  int sort;

  Gift({
    this.id,
    this.name,
    this.image,
    this.gif,
    this.price,
    this.popularity,
    this.luck_rate,
    this.type,
    this.sort,
  });

  static Gift fromJson(Map<String, dynamic> json) {
    final bean = Gift();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.image = json['image'] as String;
    bean.gif = json['gif'] as String;
    bean.price = json['price'] as int;
    bean.popularity = json['popularity'] as int;
    bean.luck_rate = json['luck_rate'] as String;
    bean.type = json['type'] as int;
    bean.sort = json['sort'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'image': image,
      'gif': gif,
      'price': price,
      'popularity': popularity,
      'luck_rate': luck_rate,
      'type': type,
      'sort': sort,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 直播间
class LiveRoom {
  int id;
  String url;
  String name;
  int audience_num;
  int category_id;
  String category_name;
  String room_no;
  String thumb;
  String announcement;
  int popularity;
  String introduction;
  String province;
  String city;
  String county;
  String location;
  String stream;
  int screen;
  int is_living;
  int type;
  String start_time;
  int duration;
  int pk_id;
  String pk_stream;
  int guard_num;
  int speak_level;
  int bullet_level;
  int level;
  int exp;
  int exp_max;
  String play_url;
  Anchor anchor;
  LiveCategory category;
  Map<String, dynamic> audience;

  LiveRoom({
    this.id,
    this.url,
    this.name,
    this.audience_num,
    this.category_id,
    this.category_name,
    this.room_no,
    this.thumb,
    this.announcement,
    this.popularity,
    this.introduction,
    this.province,
    this.city,
    this.county,
    this.location,
    this.stream,
    this.screen,
    this.is_living,
    this.type,
    this.start_time,
    this.duration,
    this.pk_id,
    this.pk_stream,
    this.guard_num,
    this.speak_level,
    this.bullet_level,
    this.level,
    this.exp,
    this.exp_max,
    this.play_url,
    this.anchor,
    this.category,
    this.audience,
  });

  static LiveRoom fromJson(Map<String, dynamic> json) {
    final bean = LiveRoom();
    bean.id = json['id'] as int;
    bean.url = json['url'] as String;
    bean.name = json['name'] as String;
    bean.audience_num = json['audience_num'] as int;
    bean.category_id = json['category_id'] as int;
    bean.category_name = json['category_name'] as String;
    bean.room_no = json['room_no'] as String;
    bean.thumb = json['thumb'] as String;
    bean.announcement = json['announcement'] as String;
    bean.popularity = json['popularity'] as int;
    bean.introduction = json['introduction'] as String;
    bean.province = json['province'] as String;
    bean.city = json['city'] as String;
    bean.county = json['county'] as String;
    bean.location = json['location'] as Null;
    bean.stream = json['stream'] as String;
    bean.screen = json['screen'] as int;
    bean.is_living = json['is_living'] as int;
    bean.type = json['type'] as int;
    bean.start_time = json['start_time'] as String;
    bean.duration = json['duration'] as int;
    bean.pk_id = json['pk_id'] as int;
    bean.pk_stream = json['pk_stream'] as String;
    bean.guard_num = json['guard_num'] as int;
    bean.speak_level = json['speak_level'] as Null;
    bean.bullet_level = json['bullet_level'] as Null;
    bean.level = json['level'] as int;
    bean.exp = json['exp'] as int;
    bean.exp_max = json['exp_max'] as int;
    bean.play_url = json['play_url'] as String;
    bean.anchor =
        json['anchor'] != null ? Anchor.fromJson(json['anchor']) : null;
    bean.category = json['category'] != null
        ? LiveCategory.fromJson(json['category'])
        : null;
    bean.audience = json['audience'] as Map<String, dynamic>;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'url': url,
      'name': name,
      'audience_num': audience_num,
      'category_id': category_id,
      'category_name': category_name,
      'room_no': room_no,
      'thumb': thumb,
      'announcement': announcement,
      'popularity': popularity,
      'introduction': introduction,
      'province': province,
      'city': city,
      'county': county,
      'location': location,
      'stream': stream,
      'screen': screen,
      'is_living': is_living,
      'type': type,
      'start_time': start_time,
      'duration': duration,
      'pk_id': pk_id,
      'pk_stream': pk_stream,
      'guard_num': guard_num,
      'speak_level': speak_level,
      'bullet_level': bullet_level,
      'level': level,
      'exp': exp,
      'exp_max': exp_max,
      'play_url': play_url,
      'anchor': anchor,
      'category': category,
      'audience': audience,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 观众
class Audience {
  int id;
  int room_id;
  int user_id;
  int is_admin;
  int level;
  int is_guard;
  Null guard_expired_at;
  int is_vip;
  int vip_level;
  Null vip_expired_at;
  int consume;
  int contribution;
  int exp;
  int intimacy;
  int is_banned;
  Null baned_expired_at;
  String visit_at;
  String created_at;
  String updated_at;
  Null admin_at;
  int no_speak;
  Null no_speak_expired_at;

  Audience({
    this.id,
    this.room_id,
    this.user_id,
    this.is_admin,
    this.level,
    this.is_guard,
    this.guard_expired_at,
    this.is_vip,
    this.vip_level,
    this.vip_expired_at,
    this.consume,
    this.contribution,
    this.exp,
    this.intimacy,
    this.is_banned,
    this.baned_expired_at,
    this.visit_at,
    this.created_at,
    this.updated_at,
    this.admin_at,
    this.no_speak,
    this.no_speak_expired_at,
  });

  static Audience fromJson(Map<String, dynamic> json) {
    final bean = Audience();
    bean.id = json['id'] as int;
    bean.room_id = json['room_id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.is_admin = json['is_admin'] as int;
    bean.level = json['level'] as int;
    bean.is_guard = json['is_guard'] as int;
    bean.guard_expired_at = json['guard_expired_at'] as Null;
    bean.is_vip = json['is_vip'] as int;
    bean.vip_level = json['vip_level'] as int;
    bean.vip_expired_at = json['vip_expired_at'] as Null;
    bean.consume = json['consume'] as int;
    bean.contribution = json['contribution'] as int;
    bean.exp = json['exp'] as int;
    bean.intimacy = json['intimacy'] as int;
    bean.is_banned = json['is_banned'] as int;
    bean.baned_expired_at = json['baned_expired_at'] as Null;
    bean.visit_at = json['visit_at'] as String;
    bean.created_at = json['created_at'] as String;
    bean.updated_at = json['updated_at'] as String;
    bean.admin_at = json['admin_at'] as Null;
    bean.no_speak = json['no_speak'] as int;
    bean.no_speak_expired_at = json['no_speak_expired_at'] as Null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'room_id': room_id,
      'user_id': user_id,
      'is_admin': is_admin,
      'level': level,
      'is_guard': is_guard,
      'guard_expired_at': guard_expired_at,
      'is_vip': is_vip,
      'vip_level': vip_level,
      'vip_expired_at': vip_expired_at,
      'consume': consume,
      'contribution': contribution,
      'exp': exp,
      'intimacy': intimacy,
      'is_banned': is_banned,
      'baned_expired_at': baned_expired_at,
      'visit_at': visit_at,
      'created_at': created_at,
      'updated_at': updated_at,
      'admin_at': admin_at,
      'no_speak': no_speak,
      'no_speak_expired_at': no_speak_expired_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 套餐
class Combo {
  int id;
  String name;
  String original_price;
  String price;
  int day;
  String created_at;
  String updated_at;
  String deleted_at;

  Combo({
    this.id,
    this.name,
    this.original_price,
    this.price,
    this.day,
    this.created_at,
    this.updated_at,
    this.deleted_at,
  });

  static Combo fromJson(Map<String, dynamic> json) {
    final bean = Combo();
    bean.id = json['id'] as int;
    bean.name = json['name'] as String;
    bean.original_price = json['original_price'] as String;
    bean.price = json['price'] as String;
    bean.day = json['day'] as int;
    bean.created_at = json['created_at'] as String;
    bean.updated_at = json['updated_at'] as String;
    bean.deleted_at = json['deleted_at'] as Null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'name': name,
      'original_price': original_price,
      'price': price,
      'day': day,
      'created_at': created_at,
      'updated_at': updated_at,
      'deleted_at': deleted_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class Gateway {
  String default_;
  List<PayAgent> list;

  Gateway({
    this.default_,
    this.list,
  });

  static Gateway fromJson(Map<String, dynamic> json) {
    final bean = Gateway();
    bean.default_ = json['default'] as String;
    if (json['list'] != null) {
      bean.list = List<PayAgent>();
      json['list'].forEach((v) {
        bean.list.add(PayAgent.fromJson(v as Map<String, dynamic>));
      });
    }
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'default': default_,
      'list': list,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class PayAgent {
  String gateway;
  String type;
  String name;

  PayAgent({
    this.gateway,
    this.type,
    this.name,
  });

  static PayAgent fromJson(Map<String, dynamic> json) {
    final bean = PayAgent();
    bean.gateway = json['gateway'] as String;
    bean.type = json['type'] as String;
    bean.name = json['name'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'gateway': gateway,
      'type': type,
      'name': name,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class PayRule {
  RechargeRates recharge_rates;
  int min_recharge;
  List<int> default_prices;
  Gateway gateways;

  PayRule({
    this.recharge_rates,
    this.min_recharge,
    this.default_prices,
    this.gateways,
  });

  static PayRule fromJson(Map<String, dynamic> json) {
    final bean = PayRule();
    bean.recharge_rates = json['recharge_rates'] != null
        ? RechargeRates.fromJson(json['recharge_rates'])
        : null;
    bean.min_recharge = json['min_recharge'] as int;
    bean.default_prices = json['default_prices'].cast<int>();
    bean.gateways =
        json['gateways'] != null ? Gateway.fromJson(json['gateways']) : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'recharge_rates': recharge_rates,
      'min_recharge': min_recharge,
      'default_prices': default_prices,
      'gateways': gateways,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class RechargeRates {
  int gold;
  int diamond;

  RechargeRates({
    this.gold,
    this.diamond,
  });

  static RechargeRates fromJson(Map<String, dynamic> json) {
    final bean = RechargeRates();
    bean.gold = json['gold'] as int;
    bean.diamond = json['diamond'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'gold': gold,
      'diamond': diamond,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 我的评论
class MyCommentedActivity {
  int id;
  int user_id;
  int topic_id;
  String content;
  List<String> pictures;
  Link link;
  int lat;
  int lng;
  int read_num;
  int distance;
  String good_count;
  int comments_amount;
  String created_at;
  int is_good;
  User user;
  List<Comment> comments;
  List<String> thumb_pictures;

  MyCommentedActivity({
    this.id,
    this.user_id,
    this.topic_id,
    this.content,
    this.pictures,
    this.link,
    this.lat,
    this.lng,
    this.read_num,
    this.distance,
    this.good_count,
    this.comments_amount,
    this.created_at,
    this.is_good,
    this.user,
    this.comments,
    this.thumb_pictures,
  });

  static MyCommentedActivity fromJson(Map<String, dynamic> json) {
    final bean = MyCommentedActivity();
    bean.id = json['id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.topic_id = json['topic_id'] as int;
    bean.content = json['content'] as String;
    bean.pictures = json['pictures'] as Null;
    bean.link = json['link'] as Null;
    bean.lat = json['lat'] as int;
    bean.lng = json['lng'] as int;
    bean.read_num = json['read_num'] as int;
    bean.distance = json['distance'] as int;
    bean.good_count = json['good_count'] as Null;
    bean.comments_amount = json['comments_amount'] as int;
    bean.created_at = json['created_at'] as String;
    bean.is_good = json['is_good'] as int;
    bean.user = json['user'] != null ? User.fromJson(json['user']) : null;
    if (json['comments'] != null) {
      bean.comments = List<Comment>();
      json['comments'].forEach((v) {
        bean.comments.add(Comment.fromJson(v as Map<String, Object>));
      });
    }
    bean.thumb_pictures = json['thumb_pictures'] as Null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'user_id': user_id,
      'topic_id': topic_id,
      'content': content,
      'pictures': pictures,
      'link': link,
      'lat': lat,
      'lng': lng,
      'read_num': read_num,
      'distance': distance,
      'good_count': good_count,
      'comments_amount': comments_amount,
      'created_at': created_at,
      'is_good': is_good,
      'user': user,
      'comments': comments,
      'thumb_pictures': thumb_pictures,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 热门关键词
class HotKeyword {
  String keyword;
  int count;

  HotKeyword({
    this.keyword,
    this.count,
  });

  static HotKeyword fromJson(Map<String, dynamic> json) {
    final bean = HotKeyword();
    bean.keyword = json['keyword'] as String;
    bean.count = json['count'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'keyword': keyword,
      'count': count,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

//region 账单 统一了 充值 直播收入 提现 会员购买
class Bill {
  int id;
  int type;

  /// 人民币
  String amount;
  String desc;
  String category;
  String created_at;
  Relation relation;

  Bill({
    this.id,
    this.type,
    this.amount,
    this.desc,
    this.category,
    this.created_at,
    this.relation,
  });

  static Bill fromJson(Map<String, dynamic> json) {
    final bean = Bill();
    bean.id = json['id'] as int;
    bean.type = json['type'] as int;
    bean.amount = json['amount'] as String;
    bean.desc = json['desc'] as String;
    bean.category = json['category'] as String;
    bean.created_at = json['created_at'] as String;
    bean.relation =
        json['relation'] != null ? Relation.fromJson(json['relation']) : null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'type': type,
      'amount': amount,
      'desc': desc,
      'category': category,
      'created_at': created_at,
      'relation': relation,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class Relation {
  String amount;
  String remark;
  int gold;

  Relation({
    this.amount,
    this.remark,
    this.gold,
  });

  static Relation fromJson(Map<String, dynamic> json) {
    final bean = Relation();
    bean.amount = json['amount'] as String;
    bean.remark = json['remark'] as String;
    bean.gold = json['gold'] as int;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'amount': amount,
      'remark': remark,
      'gold': gold,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}
//endregion

/// 提现账户
class WithdrawAccount {
  int id;
  int user_id;
  String account;
  String account_name;
  String created_at;
  String updated_at;

  WithdrawAccount({
    this.id,
    this.user_id,
    this.account,
    this.account_name,
    this.created_at,
    this.updated_at,
  });

  static WithdrawAccount fromJson(Map<String, dynamic> json) {
    final bean = WithdrawAccount();
    bean.id = json['id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.account = json['account'] as String;
    bean.account_name = json['account_name'] as String;
    bean.created_at = json['created_at'] as String;
    bean.updated_at = json['updated_at'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'user_id': user_id,
      'account': account,
      'account_name': account_name,
      'created_at': created_at,
      'updated_at': updated_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

/// 动态消息推送设置
class NotificationSetting {
  int id;
  int user_id;
  int all;
  int sound;
  int quake;
  int good;
  int comment;
  int follow;
  int subscribe;
  int share;
  int system;
  String created_at;
  String updated_at;

  NotificationSetting({
    this.id,
    this.user_id,
    this.all,
    this.sound,
    this.quake,
    this.good,
    this.comment,
    this.follow,
    this.subscribe,
    this.share,
    this.system,
    this.created_at,
    this.updated_at,
  });

  static NotificationSetting fromJson(Map<String, dynamic> json) {
    final bean = NotificationSetting();
    bean.id = json['id'] as int;
    bean.user_id = json['user_id'] as int;
    bean.all = json['all'] as int;
    bean.sound = json['sound'] as int;
    bean.quake = json['quake'] as int;
    bean.good = json['good'] as int;
    bean.comment = json['comment'] as int;
    bean.follow = json['follow'] as int;
    bean.subscribe = json['subscribe'] as int;
    bean.share = json['share'] as int;
    bean.system = json['system'] as int;
    bean.created_at = json['created_at'] as String;
    bean.updated_at = json['updated_at'] as String;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'id': id,
      'user_id': user_id,
      'all': all,
      'sound': sound,
      'quake': quake,
      'good': good,
      'comment': comment,
      'follow': follow,
      'subscribe': subscribe,
      'share': share,
      'system': system,
      'created_at': created_at,
      'updated_at': updated_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}

class Order {
  String order_no;
  int status;
  int price;
  String remark;
  String created_at;
  String payment_type;
  String success_at;

  Order({
    this.order_no,
    this.status,
    this.price,
    this.remark,
    this.created_at,
    this.payment_type,
    this.success_at,
  });

  static Order fromJson(Map<String, dynamic> json) {
    final bean = Order();
    bean.order_no = json['order_no'] as String;
    bean.status = json['status'] as int;
    bean.price = json['price'] as int;
    bean.remark = json['remark'] as String;
    bean.created_at = json['created_at'] as String;
    bean.payment_type = json['payment_type'] as Null;
    bean.success_at = json['success_at'] as Null;
    return bean;
  }

  Map<String, Object> toJson() {
    return {
      'order_no': order_no,
      'status': status,
      'price': price,
      'remark': remark,
      'created_at': created_at,
      'payment_type': payment_type,
      'success_at': success_at,
    };
  }

  @override
  String toString() {
    return JsonEncoder.withIndent('  ').convert(toJson());
  }
}
